// thanks to nem for sharing the battery stuff on ps2dev
// thanks to Chilly Willy for the source this is based on
//
// overwrite serial number to 0xffffffff to make service mode battery
// overwrite serial number to 0x00000000 to make autoboot mode battery
// serial number is stored at address 0x07 and address 0x09

#include <pspsdk.h>
#include <pspkernel.h>
#include <string.h>
#include <pspsysmem.h>

PSP_MODULE_INFO("BatSer", 0x1006, 1, 2);
PSP_MAIN_THREAD_ATTR(0);

// ** syscon prototypes **
// syscon functions for up to v3.60
// text 0x13 - write battery EEPROM	addr:00..7f  data:0000..ffff		return error code
u32 sceSyscon_driver_1165C864(u8 addr, u16 data);
// text 0x14 - read battery EEPROM		addr:00..7f 	return error code or data(0000xxxx) OR 0x802500b8 : battery not in place
u32 sceSyscon_driver_68EF0BEF(u8 addr);
// syscon functions for 3.71
u32 sceSyscon_writebat371 (u8 addr, u16 data);
u32 sceSyscon_readbat371 (u8 addr);

// function pointers
u32 (* writeBat) (u8 addr, u16 data);
u32 (* readBat) (u8 addr);

int RetVer(void)
{
	u32 k1, ret;
	k1 = pspSdkSetK1(0);
	ret = sceKernelDevkitVersion();
	pspSdkSetK1(k1);
	return ret;
}	

int errCheck(u32 data)
{
	if (data == 0x802500b8)
		return -1; // battery not in place or not ready
	else if (data & 0xffff0000)
		return ((data & 0xffff0000)>>16);
	return 0;
}

// read the serial data locations
int ReadSerial(u16* pdata)
{
	int err = 0;
	u32 k1, data;

	k1 = pspSdkSetK1(0);

	// serial number is stored at address 0x07 and address 0x09
//	data = sceSyscon_driver_68EF0BEF(0x07); // lower 16bit
	data = readBat(0x07); // lower 16bit
	err = errCheck(data);
	if (!err)
	{
		pdata[0] = (data &0xffff);
//		data = sceSyscon_driver_68EF0BEF(0x09); // upper 16bit
		data = readBat(0x09); // upper 16bit
		err = errCheck(data);
		if (!err)
			pdata[1] =  (data &0xffff);
	}

	pspSdkSetK1(k1);
	return err;
}

// write the serial data locations
int WriteSerial(u16* pdata)
{
	int err = 0;
	u32 k1;

	k1 = pspSdkSetK1(0);

	err = writeBat(0x07, pdata[0]); // lower 16bit
	if (!err)
		err = writeBat(0x09, pdata[1]); // lower 16bit

	pspSdkSetK1(k1);
	return err;
}

// read anywhere in the entire prom
int ReadProm(int address, u16* pdata)
{
	int err = 0;
	u32 k1, data;

	k1 = pspSdkSetK1(0);

	// read the data at address
	// serial number is stored at address 0x07 and address 0x09
	data = readBat(address); // lower 16bit
	err = errCheck(data);
	if (!err)
	{
		pdata[0] = (data &0xffff);
	}

	pspSdkSetK1(k1);
	return err;
}

// write anywhere in the entire prom
int WriteProm(int address, u16* pdata)
{
	int err = 0;
	u32 k1;

	k1 = pspSdkSetK1(0);

	err = writeBat(address, pdata[0]);

	pspSdkSetK1(k1);
	return err;
}

int module_start(SceSize args, void *argp)
{
	u32 k1;
	int vers, ret = 0;
	// setup the functions
	k1 = pspSdkSetK1(0);
	vers = sceKernelDevkitVersion();
	if (vers < 0x03070110) // less than 3.71 ~ 3.70 was never set as homebrew-enabled afaik
	{
		writeBat = sceSyscon_driver_1165C864;
		readBat = sceSyscon_driver_68EF0BEF;
	}
	else if (vers == 0x03070110) // if FW is 3.71
	{
		writeBat = sceSyscon_writebat371;
		readBat = sceSyscon_readbat371;
	}
	else
		ret = -1;
	pspSdkSetK1(k1);
	return ret;
}

int module_stop()
{
	return 0;
}
